@@ -168,8 +168,8 @@ git submodule update --init
 .fzf/install --all
 curl -fLo ~/.config/nvim/autoload/plug.vim --create-dirs https://raw.githubusercontent.com/junegunn/vim-plug/master/plug.vim
 curl -fLo ~/.vim/autoload/plug.vim --create-dirs https://raw.githubusercontent.com/junegunn/vim-plug/master/plug.vim
-vim -u ~/.vimrc_back "+call plug#begin()" +PlugInstall +qa &
-( sleep 60; nvim -Vnvim_log -u ~/.vimrc_back "+call plug#begin()" +PlugInstall +checkhealth +qa )&
+# vim -u ~/.vimrc_back "+call plug#begin()" +PlugInstall +qa &
+# ( sleep 60; nvim -Vnvim_log -u ~/.vimrc_back "+call plug#begin()" +PlugInstall +checkhealth +qa )&
 ln -s .shrc_customised.macos .shrc_customised
 echo "alias gdrive='gdrive  --service-account a.json'" >> ~/.bash_aliases
 echo "unalias vim" >> ~/.bash_aliases
@@ -209,6 +209,7 @@ OLDPWD=/root
 # color_my_prompt
 locale-gen
 echo "#" $(grep 'cpu ' /proc/stat >/dev/null;sleep 0.1;grep 'cpu ' /proc/stat | awk -v RS="" '{print "CPU: "($13-$2+$15-$4)*100/($13-$2+$15-$4+$16-$5)"%"}') "Mem: "$(awk '/MemTotal/{t=$2}/MemAvailable/{a=$2}END{print 100-100*a/t"%"}' /proc/meminfo) "Uptime: "$(uptime | awk '{print $1 " " $2 " " $3}')
+echo "#" $TPU_NAME
 conda activate base
 EOF
 }
@@ -271,11 +272,20 @@ shift
 ORIG_PORT=23454
 
 CHECK_PORT=$((ORIG_PORT + 1))
-apt update && apt install -y netcat nmap screen time locales
-apt install -y mosh fish tig ctags htop tree pv tmux psmisc neovim expect &
 conda install -y -c eumetsat expect & # https://askubuntu.com/questions/1047900/unbuffer-stopped-working-months-ago
+apt update && apt install -y netcat nmap screen time locales >/dev/null 2>&1
+apt install -y mosh fish tig ctags htop tree pv tmux psmisc neovim expect >/dev/null 2>&1 &
+
+conda init bash
+cat >> ~/.bashrc << EOF
+conda activate base # as my dotfiles will fiddle with conda
+export SERVER=$SERVER
+export CHECK_PORT=$CHECK_PORT
+EOF
 
 source rpt # rvs IDE env setup
+export SERVER=$SERVER
+export CHECK_PORT=$CHECK_PORT
 
 wait_ncat() {
     wait_for_ncat=$1
@@ -291,8 +301,8 @@ which $NC >/dev/null || NC=nc
 export NC
 
 pip install ripdb pydicom parse pytest-logger python_logging_rabbitmq coverage &
-python3 -m pip install pyvim neovim msgpack==1.0.0 &
-python -m pip install pyvim neovim msgpack==1.0.0 & # for vim
+# python3 -m pip install pyvim neovim msgpack==1.0.0 &
+# python -m pip install pyvim neovim msgpack==1.0.0 & # for vim
 
 SRC_WORK_FOLDER=/kaggle/working
 [ -d ${SRC_WORK_FOLDER} ] || mkdir -p ${SRC_WORK_FOLDER}
@@ -300,6 +310,8 @@ SRC_WORK_FOLDER=/kaggle/working
 cd ${SRC_WORK_FOLDER}
 
 if [ -d ${REPO} ]; then rm -rf ${REPO}; fi
+
+# get code
 {
     mvdir() {
         [[ "$2"/"$1" -ef "${PWD}" ]] || {
@@ -317,38 +329,49 @@ if [ -d ${REPO} ]; then rm -rf ${REPO}; fi
     find . -maxdepth 1 -name ".??*" -o -name "??*" -type f | xargs -I{} mv {} $OLDPWD
     find . -maxdepth 1 -name ".??*" -o -name "??*" -type d | xargs -I{} bash -x -c "mvdir {}  $OLDPWD"
     popd
-    pip install -e .
+    pip install -e . &
+    make install_dep >/dev/null
 }
 
-USE_AMQP=0 # just plain old good netcat
+USE_AMQP=true
 export USE_AMQP
 
+conda init bash
+source ~/.bashrc
+conda activate base
+
 if [ x"${PHASE}" = x"dev" ]; then
     export PS4='[Remote]: Line ${LINENO}: '
-
     (
         echo "MOSHing"
-        make mosh &
+        make mosh
     ) &
 
     if [ "x${ENABLE_RVS}" = x1 ]; then
+        make toxic | if [ $USE_AMQP -eq true ]; then cat -; else $NC --send-only -w 120s -i $((60 * 5))s $SERVER $CHECK_PORT; fi &
+
         if [ -z $(pgrep -f 'jupyter-notebook') ]; then
             bash ./rvs.sh $SERVER $PORT 2>&1 &
         else
             screen -d -m bash -c "{ echo [REMOTE]: rvs log below.; bash -x ./rvs.sh $SERVER $PORT 2>&1; } | $NC --send-only --no-shutdown -w 120s -i $((3600 * 2))s $SERVER $CHECK_PORT"
         fi
     fi &
-    conda activate base
-    make install_dep >/dev/null
-    unbuffer make toxic 2>&1 | unbuffer -p tee -a toxic_log | ([ $USE_AMQP -eq 0 ] && unbuffer -p $NC --send-only -w 120s -i $((60 * 5))s $SERVER $CHECK_PORT || unbuffer -p cat -)
+    wait # not exit, when dev
 fi
 
-if [ x"${PHASE}" != x"dev" ]; then
-    pip install kaggle_runner
-    python main.py "$@"
+if [ x"${PHASE}" = x"data" ]; then
+    bash ./rvs.sh $SERVER $PORT >/dev/null & make m & # just keep one rvs incase
+    make
 fi
 
-wait # not exit
+if [ x"${PHASE}" = x"run" ]; then
+    #pip install kaggle_runner
+    bash ./rvs.sh $SERVER $PORT >/dev/null & make m & # just keep one rvs incase
+    make toxic | if [ $USE_AMQP -eq true ]; then cat -; else $NC --send-only -w 120s -i $((60 * 5))s $SERVER $CHECK_PORT; fi
+    # basically the reverse of the calling path
+    pkill make & pkill -f "mosh" & pkill sleep & pkill -f "rvs.sh" & pkill ncat &
+    # python main.py "$@"
+fi
 """
 
 
@@ -368,9 +391,10 @@ class Coordinator:
     @staticmethod
     def push(runner):
         "Push the code to server/kagger docker"
-        logger.debug(" ".join(["kaggle", "kernels", "push", "-p", runner]))
+        comm = f"PATH=~/.local/bin:$PATH kaggle kernels push -p {runner}"
+        logger.debug(comm)
 
-        return subprocess.run(["kaggle", "kernels", "push", "-p", runner])
+        return subprocess.run(comm, shell=True)
 
     def push_listen(self):
         self.push_all()
@@ -407,13 +431,20 @@ class Coordinator:
             jf.truncate()
 
     @staticmethod
-    def _change_main_py(path, size, net, AMQPURL, seed, port, gdrive_enable=False):
+    def _change_main_py(path, size, net, AMQPURL, seed, port, gdrive_enable=False, phase='dev'):
         s = Template(
             """#!/usr/bin/env python3
+from importlib import reload
 import selectors
 import subprocess
 import sys
 
+subprocess.run('git clone https://github.com/pennz/kaggle_runner; rsync -r kaggle_runner/.* .; rsync -r kaggle_runner/* .; python -m pip install -e .', shell=True, check=True)
+import kaggle_runner
+reload(kaggle_runner)
+from kaggle_runner import logger
+logger.debug("Logger loaded")
+
 with open("runner.sh", "w") as f:
     f.write(
 r\"\"\"${runner_src}\"\"\"
@@ -431,17 +462,7 @@ with open("gdrive_setup", "w") as f:
 r\"\"\"${gdrive_str}\"\"\"
     )
 entry_str = r\"\"\"#!/bin/bash
-#nc vtool.duckdns.org 9000 -c 'python -c "import pty; pty.spawn([\"/bin/bash\",\"-li\"]);"'
-
-#apt install netcat -y
-#git clone  https://github.com/pennz/kaggle_runner
-#rsync -r kaggle_runner/* .
-#        subprocess.run(f"python ./kaggle_runner/runners/coordinator.py "
-#                       f"{runner_configs[1]['port']}", shell=True, check=True)
-#	$(PY3) -m pytest -k "test_generate_runner" tests/test_coord.py; cd .runners/intercept-resnet-384/ && $(PY3) main.py
-#make test
-
-PS4='Line ${LINENO}: ' bash -x runner.sh pennz kaggle_runner master dev 1 "vtool.duckdns.org" "$port" "$AMQPURL" "$size" "$seed" "$network" >>runner_log
+PS4='Line ${LINENO}: ' bash -x runner.sh pennz kaggle_runner master "$phase" 1 "pengyuzhou.com" "$port" "$AMQPURL" "$size" "$seed" "$network" | tee runner_log
 \"\"\"
 if ${gdrive_enable}:
     entry_str += r\"\"\"PS4='Line ${LINENO}: ' bash -x gdrive_setup >>loggdrive &\"\"\"
@@ -461,16 +482,17 @@ sel.register(p.stderr, selectors.EVENT_READ)
 
 while True:
    for key, _ in sel.select():
-       data = key.fileobj.read1(100).decode()
+       data = key.fileobj.read1(1024).decode()
        if not data:
            exit()
+       data = data.strip()
        if data == "":
            continue
        if key.fileobj is p.stdout:
-           #logger.debug(data)
+           logger.debug(data)
            print(data, end="")
        else:
-           #logger.error(data)
+           logger.error(data)
            print(data, end="", file=sys.stderr)
 
 # URL:
@@ -522,7 +544,8 @@ while True:
             network=net,
             seed=seed,
             gdrive_enable=gdrive_enable,
-            port=port
+            port=port,
+            phase=phase
         )
         ss = s.safe_substitute(d)
 
@@ -541,17 +564,18 @@ while True:
         name = net.replace("_", "-") + "-" + str(size)
         AMQPURL = config["AMQPURL"]
         port = config["port"]
+        phase = config.get("phase","dev")
 
         path = os.path.join(self.tmp_path, name)
         shutil.copytree(self.template_path, path)
 
         if from_template:
             self._change_kernel_meta_info(path, None, script)
-            self._change_main_py(path, size, net, AMQPURL, seed, port)
+            self._change_main_py(path, size, net, AMQPURL, seed, port, phase=phase)
         else:
             self._change_kernel_meta_info(
                 path, self.title_prefix + " " + name, script)
-            self._change_main_py(path, size, net, AMQPURL, seed, port)
+            self._change_main_py(path, size, net, AMQPURL, seed, port, phase=phase)
 
         if not script:
             subprocess.run(
@@ -564,12 +588,14 @@ while True:
 
 if __name__ == "__main__":
     port = sys.argv[1]
+    assert int(port) >= 0
+    phase = sys.argv[2]
     logger.debug(f"{sys.argv}")
     tmp_path = '.r'
 
-    subprocess.run(f"rm -r {tmp_path}", shell=True, check=True)
+    subprocess.run(f"rm -rf {tmp_path}", shell=True, check=True)
     coordinator = Coordinator(tmp_path, "Test Runner")
-    config = {"port":port, "size": 384, "network": "intercept", "AMQPURL": AMQPURL()}
+    config = {"phase": phase, "port":port, "size": 384, "network": "intercept", "AMQPURL": AMQPURL()}
     path = coordinator.create_runner(config, 19999, False)
 
     if os.getenv("CI") != "true":
