@@ -1,15 +1,25 @@
-#!/bin/bash
+#!/bin/bash -x
 # trap ctrl-c and call ctrl_c()
 PS4='L${LINENO}: '
 
 mosh=
+phase=
 
 if [ $# -gt 0 ]; then
-    mosh=true
+    if [ "$1" = mosh ]; then
+        mosh=true
+    else
+        phase=$1
+        if [ x$phase == xprod ]; then # production mode won't connect back
+            echo 0
+            exit 0
+        fi
+    fi
 fi
 
 trap ctrl_c INT
 NC=ncat
+trial_cnt=0
 
 function ctrl_c() {
     echo >&2 "** Trapped CTRL-C"
@@ -24,16 +34,30 @@ getNewPort() {
     else
         newNode=9000
     fi
-    unbuffer echo $newNode >>$serverNodes
+    echo $newNode >>$serverNodes
     echo -n $newNode
 }
 
 mosh_connect() {
     local newPort=$1
+    trial_cnt=$((trial_cnt + 1))
 
     pgrep -f "ncat.*p $newPort" >/dev/null && return 1 # port used
-    ~/bin/upnp-add-port $newPort UDP >/dev/null 2>&1   # port forward, rvs will connect to this port
-    ret=$?
+    #~/bin/upnp-add-port $newPort UDP >/dev/null 2>&1   # port forward, rvs will connect to this port
+    # type -f firewall-cmd >/dev/null 2>&1 && { sudo firewall-cmd --add-port $newPort/udp --permanent && sudo firewall-cmd --reload >/dev/null 2>&1;
+    type -f firewall-cmd >/dev/null 2>&1 && { sudo firewall-cmd --add-port $newPort/udp >/dev/null 2>&1;
+        ret=$?
+    } || ret=0
+
+    if [ $trial_cnt -gt 4 ]; then
+        echo >&2 connection error thing
+        exit 1 #
+    fi
+
+    (
+        sleep 1
+        bash pcc $newPort >/dev/null 2>&1
+    ) &
 
     # echo "" # blank message, to activate? will make it fail?
     $NC -ulp $newPort
@@ -44,15 +68,27 @@ mosh_connect() {
 
 connect() {
     local newPort=$1
+    trial_cnt=$((trial_cnt + 1))
 
-    pgrep -f "lp $newPort" >/dev/null && return 1 # port used
-    ~/bin/upnp-add-port $newPort                  # port forward, rvs will connect to this port
-    ret=$?
+    pgrep -f "ncat.*p $newPort" >/dev/null && return 1 # port used
+    #~/bin/upnp-add-port $newPort                  # port forward, rvs will connect to this port
+    # type -f firewall-cmd >/dev/null 2>&1 && { sudo firewall-cmd --add-port $newPort/tcp --permanent && sudo firewall-cmd --reload >/dev/null 2>&1;
+    type -f firewall-cmd >/dev/null 2>&1 && { sudo firewall-cmd --add-port $newPort/tcp >/dev/null 2>&1;
+        ret=$?
+    } || ret=0
+    #ret=0 # just pass it
 
-    tmux new-window -t rvsConnector -n "$(git show --no-patch --oneline)" "stty raw -echo && { while true; do $NC -vvlp $newPort ; echo \"Disconnected, will re-listen again\"; sleep 1; done }"
-    #ncat 127.1 $newPort
+    if [ ! $ret -eq 0 ]; then
+        exit $ret
+    fi
 
-    #tmux new-window -t rvsConnector "stty raw -echo && { $NC 127.1 $newPort </dev/tty ; }"
+    if [ $trial_cnt -gt 4 ]; then
+        echo >&2 connection error thing
+        exit 1 #
+    fi
+
+    tmux >/dev/null select-window -t rvsConnector:{end}
+    tmux >/dev/null new-window -t rvsConnector:+1 -n "$(git show --no-patch --oneline)" "stty raw -echo && { while true; do $NC -vvlp $newPort ; echo \"Disconnected, will re-listen again\"; sleep 1; done }"
 
     # tcpserver waits for connections from TCP clients. For each connection, it
     # runs prog, with descriptor 0 reading from the network and descriptor 1 writing
@@ -67,7 +103,7 @@ connect() {
 
 port=$(getNewPort serverNodes)
 
-connects() {
+make_connect() {
     if [ -z $mosh ]; then
         connect $port serverNodes
     else
@@ -75,18 +111,20 @@ connects() {
     fi
 }
 
-connects
+make_connect
 
 while [ ! $? -eq 0 ]; do
-    echo "$port" >>serverNodes
     port=$((port + 1))
-    connects
+    echo "$port" >>serverNodes
+    make_connect
 done
 
 if [ -z $mosh ]; then
     echo -n "$port"
 fi
 
+# sync &
+
 # so reverse shell server named to RSS
 # so tcpserver instance named to TSins
 #   RSS output -> TSins[0]
